@@ -45,16 +45,16 @@ rb_file (const char *filename)
 	char *ret;
 	int i;
 
-	static char *paths[] =
-	{
-		SHARE_DIR "/",
-		SHARE_DIR "/glade/",
-		SHARE_DIR "/art/",
-		SHARE_DIR "/node-views/",
+	static char *paths[] = {
 		SHARE_UNINSTALLED_DIR "/",
 		SHARE_UNINSTALLED_DIR "/glade/",
 		SHARE_UNINSTALLED_DIR "/art/",
 		SHARE_UNINSTALLED_DIR "/node-views/"
+		SHARE_DIR "/",
+		SHARE_DIR "/glade/",
+		SHARE_DIR "/art/",
+		SHARE_DIR "/views/",
+		SHARE_DIR "/node-views/",
 	};
 	
 	g_assert (files != NULL);
@@ -63,11 +63,9 @@ rb_file (const char *filename)
 	if (ret != NULL)
 		return ret;
 
-	for (i = 0; i < (int) G_N_ELEMENTS (paths); i++)
-	{
+	for (i = 0; i < (int) G_N_ELEMENTS (paths); i++) {
 		ret = g_strconcat (paths[i], filename, NULL);
-		if (g_file_test (ret, G_FILE_TEST_EXISTS) == TRUE)
-		{
+		if (g_file_test (ret, G_FILE_TEST_EXISTS) == TRUE) {
 			g_hash_table_insert (files, g_strdup (filename), ret);
 			return (const char *) ret;
 		}
@@ -616,7 +614,6 @@ rb_uri_is_iradio (const char *uri)
 void
 rb_uri_handle_recursively (const char *text_uri,
 		           GFunc func,
-			   GMutex *cancel_lock,
 			   gboolean *cancelflag,
 		           gpointer user_data)
 {
@@ -637,14 +634,8 @@ rb_uri_handle_recursively (const char *text_uri,
 		GnomeVFSURI *file_uri;
 		char *file_uri_text;
 
-		if (cancel_lock) {
-			g_mutex_lock (cancel_lock);
-			if (*cancelflag) {
-				g_mutex_unlock (cancel_lock);
-				break;
-			}
-			g_mutex_unlock (cancel_lock);
-		}
+		if (cancelflag && *cancelflag)
+			break;
 
 		info = (GnomeVFSFileInfo *) l->data;
 		
@@ -658,11 +649,11 @@ rb_uri_handle_recursively (const char *text_uri,
 		if (info->type != GNOME_VFS_FILE_TYPE_REGULAR)
 		{
 			if ((info->type == GNOME_VFS_FILE_TYPE_DIRECTORY) &&
-			    (info->name) && (info->name[0] != '.'))
+			    (info->name) && (strcmp (info->name, "."))
+			    && (strcmp (info->name, "..")))
 			{
 				rb_uri_handle_recursively (file_uri_text,
 							   func,
-							   cancel_lock,
 							   cancelflag,
 							   user_data);
 
@@ -677,3 +668,28 @@ rb_uri_handle_recursively (const char *text_uri,
 	gnome_vfs_file_info_list_free (list);
 	gnome_vfs_uri_unref (uri);
 }
+
+
+GnomeVFSResult
+rb_uri_mkstemp (const char *prefix, char **uri_ret, GnomeVFSHandle **ret)
+{
+	GnomeVFSHandle *handle = NULL;
+	char *uri = NULL;
+	GnomeVFSResult result = GNOME_VFS_ERROR_FILE_EXISTS;
+	
+	
+	do {
+		g_free (uri);
+		uri = g_strdup_printf ("%s%06X", prefix, g_random_int_range (0, 0xFFFFFF));
+		result = gnome_vfs_create (&handle, uri, GNOME_VFS_OPEN_WRITE | GNOME_VFS_OPEN_RANDOM, TRUE,  0644);
+	} while (result == GNOME_VFS_ERROR_FILE_EXISTS);
+
+	if (result == GNOME_VFS_OK) {
+		*uri_ret = uri;
+		*ret = handle;
+	} else {
+		g_free (uri);
+	}
+	return result;
+}
+
