@@ -22,12 +22,9 @@
 #include <config.h>
 
 #include "rhythmdb.h"
-#include "rhythmdb-model.h"
-#include "rhythmdb-legacy.h"
 #include "rhythmdb-query-model.h"
 #include "rhythmdb-property-model.h"
-#include "monkey-media.h"
-#include "monkey-media-stream-info.h"
+#include "rb-metadata.h"
 #include <string.h>
 #include <gobject/gvaluecollector.h>
 #include <gdk/gdk.h>
@@ -53,7 +50,9 @@ struct RhythmDBPrivate
 
 	GThreadPool *query_thread_pool;
 
-	GHashTable *legacy_id_map;
+	/* Used for metadata reading */
+	RBMetaData *metadata;
+	GnomeVFSFileInfo *vfsinfo;
 
 	gboolean query_in_progress;
 
@@ -62,8 +61,13 @@ struct RhythmDBPrivate
 	GAsyncQueue *add_queue;
 	GAsyncQueue *action_queue;
 
-	GHashTable *added_entries;
-	GHashTable *changed_entries;
+	gboolean dry_run;
+
+	gboolean writelocked;
+	guint readlocks;
+
+	GList *added_entries;
+	GList *changed_entries;
 
 	GHashTable *propname_map;
 
@@ -80,6 +84,7 @@ struct RhythmDBAction
 {
 	enum {
 		RHYTHMDB_ACTION_SET,
+		RHYTHMDB_ACTION_SYNC,
 	} type;
 	RhythmDBEntry *entry;
 	guint propid;
@@ -95,6 +100,14 @@ struct RhythmDBQueryThreadData
 	gboolean cancel;
 };
 
+struct RhythmDBEntryChangeData
+{
+	RhythmDBEntry *entry;
+	RhythmDBPropType prop;
+	GValue old;
+	GValue new;
+};
+
 static void rhythmdb_class_init (RhythmDBClass *klass);
 static void rhythmdb_init (RhythmDB *source);
 static void rhythmdb_finalize (GObject *object);
@@ -117,6 +130,7 @@ enum
 {
 	PROP_0,
 	PROP_NAME,
+	PROP_DRY_RUN,
 };
 
 enum
@@ -127,7 +141,6 @@ enum
 	ENTRY_DELETED,
 	LOAD_COMPLETE,
 	ERROR,
-	LEGACY_LOAD_COMPLETE,
 	LAST_SIGNAL
 };
 
@@ -184,6 +197,13 @@ rhythmdb_class_init (RhythmDBClass *klass)
 							      NULL,
 							      G_PARAM_READWRITE));
 
+	g_object_class_install_property (object_class,
+					 PROP_DRY_RUN,
+					 g_param_spec_boolean ("dry-run",
+							       "dry run",
+							       "Whether or not changes should be saved",
+							       FALSE,
+							       G_PARAM_READWRITE));
 	rhythmdb_signals[ENTRY_ADDED] =
 		g_signal_new ("entry_added",
 			      RHYTHMDB_TYPE,
@@ -220,9 +240,9 @@ rhythmdb_class_init (RhythmDBClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (RhythmDBClass, entry_changed),
 			      NULL, NULL,
-			      g_cclosure_marshal_VOID__POINTER,
-			      G_TYPE_NONE,
-			      1, G_TYPE_POINTER);
+			      rb_marshal_VOID__POINTER_INT_POINTER_POINTER,
+			      G_TYPE_NONE, 4, G_TYPE_POINTER,
+			      G_TYPE_INT, G_TYPE_POINTER, G_TYPE_POINTER);
 
 	rhythmdb_signals[LOAD_COMPLETE] =
 		g_signal_new ("load_complete",
@@ -244,26 +264,100 @@ rhythmdb_class_init (RhythmDBClass *klass)
 			      2,
 			      G_TYPE_STRING,
 			      G_TYPE_STRING);
-	rhythmdb_signals[LEGACY_LOAD_COMPLETE] =
-		g_signal_new ("legacy-load-complete",
-			      G_OBJECT_CLASS_TYPE (object_class),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (RhythmDBClass, legacy_load_complete),
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__VOID,
-			      G_TYPE_NONE,
-			      0);
+}
+
+#if 0
+static gboolean
+prop_from_metadata_field (RBMetaDataField field, RhythmDBPropType *prop)
+{
+	switch (field) {
+	case RB_METADATA_FIELD_TITLE:
+		*prop = RHYTHMDB_PROP_TITLE;
+		return TRUE;		
+	case RB_METADATA_FIELD_ARTIST:
+		*prop = RHYTHMDB_PROP_ARTIST; 
+		return TRUE;		
+	case RB_METADATA_FIELD_ALBUM:
+		*prop = RHYTHMDB_PROP_ALBUM; 
+		return TRUE;		
+	case RB_METADATA_FIELD_GENRE:
+		*prop = RHYTHMDB_PROP_GENRE; 
+		return TRUE;		
+	case RB_METADATA_FIELD_TRACK_NUMBER:
+		*prop = RHYTHMDB_PROP_TRACK_NUMBER; 
+		return TRUE;		
+	case RB_METADATA_FIELD_DURATION:
+		*prop = RHYTHMDB_PROP_DURATION; 
+		return TRUE;		
+	case RB_METADATA_FIELD_BITRATE:
+		*prop = RHYTHMDB_PROP_BITRATE; 
+		return TRUE;		
+	default:
+		return FALSE;
+	}
+}
+#endif
+
+static gboolean
+metadata_field_from_prop (RhythmDBPropType prop, RBMetaDataField *field)
+{
+	switch (prop) {
+	case RHYTHMDB_PROP_TITLE:
+		*field = RB_METADATA_FIELD_TITLE;
+		return TRUE;		
+	case RHYTHMDB_PROP_ARTIST:
+		*field = RB_METADATA_FIELD_ARTIST; 
+		return TRUE;		
+	case RHYTHMDB_PROP_ALBUM:
+		*field = RB_METADATA_FIELD_ALBUM; 
+		return TRUE;		
+	case RHYTHMDB_PROP_GENRE:
+		*field = RB_METADATA_FIELD_GENRE; 
+		return TRUE;		
+	case RHYTHMDB_PROP_TRACK_NUMBER:
+		*field = RB_METADATA_FIELD_TRACK_NUMBER; 
+		return TRUE;		
+	case RHYTHMDB_PROP_DURATION:
+		*field = RB_METADATA_FIELD_DURATION; 
+		return TRUE;		
+	case RHYTHMDB_PROP_BITRATE:
+		*field = RB_METADATA_FIELD_BITRATE; 
+		return TRUE;		
+	case RHYTHMDB_PROP_TRACK_GAIN:
+		*field = RB_METADATA_FIELD_TRACK_GAIN;
+		return TRUE;
+	case RHYTHMDB_PROP_TRACK_PEAK:
+		*field = RB_METADATA_FIELD_TRACK_PEAK;
+		return TRUE;
+	case RHYTHMDB_PROP_ALBUM_GAIN:
+		*field = RB_METADATA_FIELD_ALBUM_GAIN;
+		return TRUE;
+	case RHYTHMDB_PROP_ALBUM_PEAK:
+		*field = RB_METADATA_FIELD_ALBUM_PEAK;
+		return TRUE;
+	default:
+		return FALSE;
+	}
 }
 
 static GType
-extract_gtype_from_enum_entry (GEnumClass *klass, guint i)
+extract_gtype_from_enum_entry (RhythmDB *db, GEnumClass *klass, guint i)
 {
 	GType ret;
 	GEnumValue *value;
+	RBMetaDataField field;
 	char *typename;
 	char *typename_end;
 	
 	value = g_enum_get_value (klass, i);
+
+	/* First check to see whether this is a property that maps to
+	   a RBMetaData property. */
+	if (metadata_field_from_prop (value->value, &field))
+		return rb_metadata_get_field_type (db->priv->metadata, field); 
+
+	/* This is a "synthetic" property. */
+
 	typename = strstr (value->value_nick, "(");
 	g_return_val_if_fail (typename != NULL, G_TYPE_INVALID);
 
@@ -291,6 +385,8 @@ rhythmdb_init (RhythmDB *db)
 	db->priv->action_queue = g_async_queue_new ();
 
 	db->priv->status_queue = g_async_queue_new ();
+
+	db->priv->metadata = rb_metadata_new ();
 	
 	prop_class = g_type_class_ref (RHYTHMDB_TYPE_PROP);
 	unsaved_prop_class = g_type_class_ref (RHYTHMDB_TYPE_UNSAVED_PROP);
@@ -302,13 +398,13 @@ rhythmdb_init (RhythmDB *db)
 	/* Now, extract the GType of each column from the enum descriptions,
 	 * and cache that for later use. */
 	for (i = 0; i < prop_class->n_values; i++) {
-		db->priv->column_types[i] = extract_gtype_from_enum_entry (prop_class, i);
+		db->priv->column_types[i] = extract_gtype_from_enum_entry (db, prop_class, i);
 		g_assert (db->priv->column_types[i] != G_TYPE_INVALID);
 	}
 			
 	
 	for (j = 0; j < unsaved_prop_class->n_values; i++, j++) {
-		db->priv->column_types[i] = extract_gtype_from_enum_entry (unsaved_prop_class, i);
+		db->priv->column_types[i] = extract_gtype_from_enum_entry (db, unsaved_prop_class, i);
 		g_assert (db->priv->column_types[i] != G_TYPE_INVALID);
 	}
 
@@ -322,8 +418,6 @@ rhythmdb_init (RhythmDB *db)
 		g_hash_table_insert (db->priv->propname_map, (gpointer) name, GINT_TO_POINTER (i));
 	}
 
-	db->priv->legacy_id_map = g_hash_table_new (NULL, NULL);
-
 	db->priv->query_thread_pool = g_thread_pool_new ((GFunc) query_thread_main,
 							 db, 1, TRUE, NULL);
 	db->priv->thread_reaper_id
@@ -391,6 +485,8 @@ rhythmdb_finalize (GObject *object)
 
 	g_return_if_fail (db->priv != NULL);
 
+	g_object_unref (db->priv->metadata);
+
 	g_static_rw_lock_free (&db->priv->lock);
 
 	g_thread_pool_free (db->priv->query_thread_pool, TRUE, FALSE);
@@ -421,6 +517,9 @@ rhythmdb_set_property (GObject *object,
 	case PROP_NAME:
 		source->priv->name = g_strdup (g_value_get_string (value));
 		break;
+	case PROP_DRY_RUN:
+		source->priv->dry_run = g_value_get_boolean (value);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -440,6 +539,9 @@ rhythmdb_get_property (GObject *object,
 	case PROP_NAME:
 		g_value_set_string (value, source->priv->name);
 		break;
+	case PROP_DRY_RUN:
+		g_value_set_boolean (value, source->priv->dry_run);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -462,37 +564,62 @@ void
 rhythmdb_write_lock (RhythmDB *db)
 {
 	g_static_rw_lock_writer_lock (&db->priv->lock);
+
+	db->priv->added_entries = NULL;
+	db->priv->changed_entries = NULL;
 }
 
 static void
-emit_entry_changed (RhythmDBEntry *entry, gpointer unused,
-		    RhythmDB *db)
+emit_changed_signals (RhythmDB *db)
 {
-	g_signal_emit (G_OBJECT (db), rhythmdb_signals[ENTRY_CHANGED], 0, entry);
+	GList *tem;
+	GHashTable *queued_entry_changes = g_hash_table_new (NULL, NULL);
+
+	for (tem = db->priv->changed_entries; tem; tem = tem->next) {
+		struct RhythmDBEntryChangeData *data = tem->data;
+		struct RhythmDBAction *action;
+		RBMetaDataField field;
+
+		if (rhythmdb_entry_get_int (db, data->entry, RHYTHMDB_PROP_TYPE) == RHYTHMDB_ENTRY_TYPE_SONG
+		    && !g_hash_table_lookup (queued_entry_changes, data->entry)
+		    && metadata_field_from_prop (data->prop, &field)) {
+			action = g_new0 (struct RhythmDBAction, 1);
+			action->type = RHYTHMDB_ACTION_SYNC;
+			action->entry = data->entry;
+			g_hash_table_insert (queued_entry_changes, data->entry, action);
+			g_async_queue_push (db->priv->action_queue, action);
+		}
+		g_signal_emit (G_OBJECT (db), rhythmdb_signals[ENTRY_CHANGED], 0, data->entry,
+			       data->prop, &data->old, &data->new);
+		g_value_unset (&data->old);
+		g_value_unset (&data->new);
+		g_free (data);
+	}
+	g_list_free (db->priv->changed_entries);
+	db->priv->changed_entries = NULL;
+	g_hash_table_destroy (queued_entry_changes);
 }
 
 static void
-emit_entry_added (RhythmDBEntry *entry, gpointer unused,
-		  RhythmDB *db)
+rhythmdb_write_unlock_internal (RhythmDB *db, gboolean commit)
 {
-	g_signal_emit (G_OBJECT (db), rhythmdb_signals[ENTRY_ADDED], 0, entry);
+	GList *tem;
+
+	if (commit)
+		emit_changed_signals (db);
+
+	for (tem = db->priv->added_entries; tem; tem = tem->next)
+		g_signal_emit (G_OBJECT (db), rhythmdb_signals[ENTRY_ADDED], 0, tem->data);
+	g_list_free (db->priv->added_entries);
+	db->priv->added_entries = NULL;
+		 
+	g_static_rw_lock_writer_unlock (&db->priv->lock);
 }
 
 void
 rhythmdb_write_unlock (RhythmDB *db)
 {
-	if (db->priv->changed_entries) {
-		g_hash_table_foreach (db->priv->changed_entries, (GHFunc) emit_entry_changed, db);
-		g_hash_table_destroy (db->priv->changed_entries);
-		db->priv->changed_entries = NULL;
-	}
-	if (db->priv->added_entries) {
-		g_hash_table_foreach (db->priv->added_entries, (GHFunc) emit_entry_added, db);
-		g_hash_table_destroy (db->priv->added_entries);
-		db->priv->added_entries = NULL;
-	}
-		 
-	g_static_rw_lock_writer_unlock (&db->priv->lock);
+	rhythmdb_write_unlock_internal (db, TRUE);
 }
 
 GQuark
@@ -508,11 +635,13 @@ rhythmdb_error_quark (void)
 static inline void
 db_enter (RhythmDB *db, gboolean write)
 {
+#ifdef RHYTHMDB_ENABLE_SANITY_CHECK
 	if (write) 
 		g_assert (db->priv->lock.have_writer);
 	else
 		g_assert (db->priv->lock.read_counter > 0
 			  || db->priv->lock.have_writer);
+#endif
 }
 
 RhythmDBEntry *
@@ -525,142 +654,178 @@ rhythmdb_entry_new (RhythmDB *db, RhythmDBEntryType type, const char *uri)
 	
 	ret = klass->impl_entry_new (db, type, uri);
 	rb_debug ("emitting entry added");
-	if (!db->priv->changed_entries)
-		db->priv->added_entries = g_hash_table_new (NULL, NULL);
 
 	if (ret != NULL)
-		g_hash_table_insert (db->priv->added_entries, ret, NULL);
+		db->priv->added_entries = g_list_append (db->priv->added_entries, ret);
 	return ret;
 }
 
-typedef struct 
+gboolean
+rhythmdb_entry_is_editable (RhythmDB *db, RhythmDBEntry *entry)
 {
-	GValue track_number_val;
-	GValue file_size_val;
-	GValue title_val;
-	GValue duration_val;
-	GValue quality_val;
-	GValue mtime_val;
-	GValue genre_val;
-	GValue artist_val;
-	GValue album_val;
-} RhythmDBEntryUpdateData;
+	return rb_metadata_can_save (db->priv->metadata,
+				     rhythmdb_entry_get_string (db, entry,
+								RHYTHMDB_PROP_MIMETYPE));
+}
 
 /* Threading: any thread
  */
-static RhythmDBEntryUpdateData *
-read_metadata (const char *location, GError **error)
+static void
+read_metadata_async (RhythmDB *db, const char *location, GError **real_error)
 {
-	RhythmDBEntryUpdateData *data = g_new0 (RhythmDBEntryUpdateData, 1);
-	MonkeyMediaStreamInfo *info;
-	GnomeVFSFileInfo *vfsinfo;
-	GValue tem = {0,};
+	GError *error = NULL;
 
-	info = monkey_media_stream_info_new (location, error);
-	if (G_UNLIKELY (info == NULL)) {
-		g_free (data);
-		return NULL;
-	}
+	rb_metadata_load (db->priv->metadata, location, &error);
+	if (error != NULL)
+		g_propagate_error (real_error, error);
 
-	/* track number */
-	if (monkey_media_stream_info_get_value (info,
-				                MONKEY_MEDIA_STREAM_INFO_FIELD_TRACK_NUMBER,
-					        0, &data->track_number_val) == FALSE) {
-		g_value_init (&data->track_number_val, G_TYPE_INT);
-		g_value_set_int (&data->track_number_val, -1);
-	}
-
-	/* duration */
-	monkey_media_stream_info_get_value (info,
-				            MONKEY_MEDIA_STREAM_INFO_FIELD_DURATION,
-					    0, &data->duration_val);
-
-	/* quality */
-	monkey_media_stream_info_get_value (info,
-				            MONKEY_MEDIA_STREAM_INFO_FIELD_AUDIO_QUALITY,
-					    0, &tem);
-	g_value_init (&data->quality_val, G_TYPE_INT);
-	g_value_set_int (&data->quality_val, g_value_get_enum (&tem));
-
-	/* filesize */
-	monkey_media_stream_info_get_value (info, MONKEY_MEDIA_STREAM_INFO_FIELD_FILE_SIZE,
-					    0, &data->file_size_val);
-
-	/* title */
-	monkey_media_stream_info_get_value (info,
-					    MONKEY_MEDIA_STREAM_INFO_FIELD_TITLE,
-					    0, &data->title_val);
-	if (*(g_value_get_string (&data->title_val)) == '\0') {
-		GnomeVFSURI *vfsuri;
-		char *fname;
-
-		vfsuri = gnome_vfs_uri_new (location);
-		fname = gnome_vfs_uri_extract_short_name (vfsuri);
-		g_value_set_string_take_ownership (&data->title_val, fname);
-		gnome_vfs_uri_unref (vfsuri);
-	}
-
-	vfsinfo = gnome_vfs_file_info_new ();
-
-	gnome_vfs_get_file_info (location, vfsinfo, GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
-
-	/* mtime */
-	g_value_init (&data->mtime_val, G_TYPE_LONG);
-	g_value_set_long (&data->mtime_val, vfsinfo->mtime);
-
-	gnome_vfs_file_info_unref (vfsinfo);
-
-	/* genre */
-	monkey_media_stream_info_get_value (info,
-				            MONKEY_MEDIA_STREAM_INFO_FIELD_GENRE,
-					    0,
-				            &data->genre_val);
-
-	/* artist */
-	monkey_media_stream_info_get_value (info,
-				            MONKEY_MEDIA_STREAM_INFO_FIELD_ARTIST,
-					    0,
-				            &data->artist_val);
-
-	/* album */
-	monkey_media_stream_info_get_value (info,
-				            MONKEY_MEDIA_STREAM_INFO_FIELD_ALBUM,
-					    0,
-				            &data->album_val);
-	g_object_unref (G_OBJECT (info));
-	return data;
+	if (db->priv->vfsinfo)
+		gnome_vfs_file_info_unref (db->priv->vfsinfo);
+	db->priv->vfsinfo = gnome_vfs_file_info_new ();
+	gnome_vfs_get_file_info (location, db->priv->vfsinfo, GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
 }
 
 static void
-synchronize_entry_with_data (RhythmDB *db, RhythmDBEntry *entry, RhythmDBEntryUpdateData *data)
+set_metadata_string_default_unknown (RhythmDB *db, RhythmDBEntry *entry,
+				     RBMetaDataField field,
+				     RhythmDBPropType prop)
 {
-	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_TRACK_NUMBER, &data->track_number_val);
-	g_value_unset (&data->track_number_val);
-	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_DURATION, &data->duration_val);
-	g_value_unset (&data->duration_val);
-	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_QUALITY, &data->quality_val);
-	g_value_unset (&data->quality_val);
-	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_FILE_SIZE, &data->file_size_val);
-	g_value_unset (&data->file_size_val);
-	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_TITLE, &data->title_val);
-	g_value_unset (&data->title_val);
-	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_MTIME, &data->mtime_val);
-	g_value_unset (&data->mtime_val);
-	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_GENRE, &data->genre_val);
-	g_value_unset (&data->genre_val);
-	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_ARTIST, &data->artist_val);
-	g_value_unset (&data->artist_val);
-	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_ALBUM, &data->album_val);
-	g_value_unset (&data->album_val);
+	const char *unknown = _("Unknown");
+	GValue val = {0, };
+	
+	if (!(rb_metadata_get (db->priv->metadata,
+			       field,
+			       &val))) {
+		g_value_init (&val, G_TYPE_STRING);
+		g_value_set_static_string (&val, unknown);
+	} else if (g_value_get_string (&val)[0] == '\0')
+		g_value_set_string (&val, unknown);
+	rhythmdb_entry_set (db, entry, prop, &val);
+	g_value_unset (&val);
+}
+
+static void
+set_props_from_metadata (RhythmDB *db, RhythmDBEntry *entry)
+{
+	const char *mime;
+	GValue val = {0,};
+
+	g_value_init (&val, G_TYPE_STRING);
+	mime = rb_metadata_get_mime (db->priv->metadata);
+	if (mime) {
+		g_value_set_string (&val, mime);
+		rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_MIMETYPE, &val);
+	}
+	g_value_unset (&val);
+
+	/* track number */
+	if (!rb_metadata_get (db->priv->metadata,
+			      RB_METADATA_FIELD_TRACK_NUMBER,
+			      &val)) {
+		g_value_init (&val, G_TYPE_INT);
+		g_value_set_int (&val, -1);
+	}
+	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_TRACK_NUMBER, &val);
+	g_value_unset (&val);
+
+	/* duration */
+	if (rb_metadata_get (db->priv->metadata,
+			     RB_METADATA_FIELD_DURATION,
+			     &val)) {
+		rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_DURATION, &val);
+		g_value_unset (&val);
+	}
+
+	/* bitrate */
+	if (rb_metadata_get (db->priv->metadata,
+			     RB_METADATA_FIELD_BITRATE,
+			     &val)) {
+		rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_BITRATE, &val);
+		g_value_unset (&val);
+	}
+
+	/* filesize */
+	g_value_init (&val, G_TYPE_UINT64);
+	g_value_set_uint64 (&val, db->priv->vfsinfo->size);
+	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_FILE_SIZE, &val);
+	g_value_unset (&val);
+
+	/* title */
+	if (!rb_metadata_get (db->priv->metadata,
+			      RB_METADATA_FIELD_TITLE,
+			      &val) || g_value_get_string (&val)[0] == '\0') {
+		char *utf8name;
+		utf8name = g_filename_to_utf8 (db->priv->vfsinfo->name, -1, NULL, NULL, NULL);
+		if (!utf8name) {
+			utf8name = g_strdup (_("<invalid filename>"));
+		}
+		g_value_init (&val, G_TYPE_STRING);
+		g_value_set_string (&val, utf8name);
+		g_free (utf8name);
+	}
+	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_TITLE, &val);
+	g_value_unset (&val);
+
+	/* mtime */
+	g_value_init (&val, G_TYPE_LONG);
+	g_value_set_long (&val, db->priv->vfsinfo->mtime);
+	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_MTIME, &val);
+	g_value_unset (&val);
+
+	gnome_vfs_file_info_unref (db->priv->vfsinfo);
+	db->priv->vfsinfo = NULL;
+
+	/* genre */
+	set_metadata_string_default_unknown (db, entry, RB_METADATA_FIELD_GENRE,
+					     RHYTHMDB_PROP_GENRE);
+
+	/* artist */
+	set_metadata_string_default_unknown (db, entry, RB_METADATA_FIELD_ARTIST,
+					     RHYTHMDB_PROP_ARTIST);
+	/* album */
+	set_metadata_string_default_unknown (db, entry, RB_METADATA_FIELD_ALBUM,
+					     RHYTHMDB_PROP_ALBUM);
+
+	/* replaygain track gain */
+        if (rb_metadata_get (db->priv->metadata,
+                             RB_METADATA_FIELD_TRACK_GAIN,
+                             &val)) {
+		rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_TRACK_GAIN, &val);
+		g_value_unset (&val);
+	}
+
+	/* replaygain track peak */
+	if (rb_metadata_get (db->priv->metadata,
+			     RB_METADATA_FIELD_TRACK_PEAK,
+			     &val)) {
+		rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_TRACK_PEAK, &val);
+		g_value_unset (&val);
+	}
+
+	/* replaygain album gain */
+	if (rb_metadata_get (db->priv->metadata,
+			     RB_METADATA_FIELD_ALBUM_GAIN,
+			     &val)) {
+		rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_ALBUM_GAIN, &val);
+		g_value_unset (&val);
+	}
+
+	/* replaygain album peak */
+	if (rb_metadata_get (db->priv->metadata,
+			     RB_METADATA_FIELD_ALBUM_PEAK,
+			     &val)) {
+		rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_ALBUM_PEAK, &val);
+		g_value_unset (&val);
+	}
 }
 
 RhythmDBEntry *
-rhythmdb_add_song (RhythmDB *db, const char *uri, GError **error)
+rhythmdb_add_song (RhythmDB *db, const char *uri, GError **real_error)
 {
 	RhythmDBEntry *entry;
 	char *realuri;
-	RhythmDBEntryUpdateData *metadata;
-	GValue last_time = {0, };
+	GError *error = NULL;
+	GValue last_time = {0, }, rating = {0, };
 
 	realuri = rb_uri_resolve_symlink (uri);
 
@@ -671,7 +836,9 @@ rhythmdb_add_song (RhythmDB *db, const char *uri, GError **error)
 	if (entry) {
 		rhythmdb_entry_ref_unlocked (db, entry);
 		rhythmdb_write_unlock (db);
-		update_song (db, entry, error);
+		update_song (db, entry, &error);
+		if (error != NULL)
+			g_propagate_error (real_error, error);
 		rhythmdb_entry_unref (db, entry);
 		return entry;
 	}
@@ -679,8 +846,9 @@ rhythmdb_add_song (RhythmDB *db, const char *uri, GError **error)
 	rhythmdb_write_unlock (db);
 
 	/* Don't do file access with the db write lock held */
-	metadata = read_metadata (uri, error);
-	if (!metadata) {
+	read_metadata_async (db, uri, &error);
+	if (error) {
+		g_propagate_error (real_error, error);
 		rb_debug ("failed to read data from \"%s\"", uri);
 		g_free (realuri);
 		return NULL;
@@ -691,7 +859,7 @@ rhythmdb_add_song (RhythmDB *db, const char *uri, GError **error)
 	entry = rhythmdb_entry_new (db, RHYTHMDB_ENTRY_TYPE_SONG, realuri);
 	if (entry == NULL)
 		goto out_dupentry;
-	synchronize_entry_with_data (db, entry, metadata);
+	set_props_from_metadata (db, entry);
 
 	/* initialize the last played date to 0=never */
 	g_value_init (&last_time, G_TYPE_LONG);
@@ -699,11 +867,16 @@ rhythmdb_add_song (RhythmDB *db, const char *uri, GError **error)
 	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_LAST_PLAYED, &last_time);
 	g_value_unset (&last_time);
 
+	/* initialize the rating to 3 */
+	g_value_init (&rating, G_TYPE_DOUBLE);
+	g_value_set_double (&rating, 3.0);
+	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_RATING, &rating);
+	g_value_unset (&rating);
+
  out_dupentry:
-	g_free (metadata);
 	g_free (realuri);
 
-	rhythmdb_write_unlock (db);
+	rhythmdb_write_unlock_internal (db, FALSE);
 	return entry;
 }
 
@@ -807,13 +980,13 @@ add_thread_main (RhythmDB *db)
 static void
 update_song (RhythmDB *db, RhythmDBEntry *entry, GError **error)
 {
-	const char *location;
+	char *location;
 	time_t stored_mtime;
 	GnomeVFSFileInfo *vfsinfo = NULL;
 	GnomeVFSResult result;
 
 	rhythmdb_read_lock (db);
-	location = rhythmdb_entry_get_string (db, entry, RHYTHMDB_PROP_LOCATION);
+	location = g_strdup (rhythmdb_entry_get_string (db, entry, RHYTHMDB_PROP_LOCATION));
 	stored_mtime = rhythmdb_entry_get_long (db, entry, RHYTHMDB_PROP_MTIME);
 	rhythmdb_read_unlock (db);
 	
@@ -838,13 +1011,15 @@ update_song (RhythmDB *db, RhythmDBEntry *entry, GError **error)
 
 	rhythmdb_write_lock (db);
 	
+	rb_debug ("song \"%s\" changed, deleting and re-adding", location);
 	rhythmdb_entry_delete (db, entry);
-	rhythmdb_add_uri_async (db, location); 
-	rhythmdb_entry_unref_unlocked (db, entry);
 
 	rhythmdb_write_unlock (db);
 
+	rhythmdb_add_uri_async (db, location); 
+
 out:
+	g_free (location);
 	gnome_vfs_file_info_unref (vfsinfo);
 }
 
@@ -893,9 +1068,46 @@ action_thread_main (RhythmDB *db)
 			rhythmdb_entry_set (db, action->entry, action->propid,
 					    &action->value);
 			rhythmdb_write_unlock (db);
+			g_value_unset (&action->value);
+			break;
+		case RHYTHMDB_ACTION_SYNC:
+		{
+			char *location;
+			GError *error = NULL;
+
+			if (db->priv->dry_run) {
+				rb_debug ("dry run is enabled, not syncing metadata");
+				break;
+			}
+			
+			rhythmdb_read_lock (db);
+			location = g_strdup (rhythmdb_entry_get_string (db, action->entry,
+									RHYTHMDB_PROP_LOCATION));
+			rhythmdb_read_unlock (db);
+			
+			rb_metadata_load (db->priv->metadata,
+					  location, &error);
+			if (error != NULL) {
+				g_warning ("error loading metadata from %s: %s", location,
+					   error->message);
+				/* FIXME */
+				g_free (location);
+				break;
+			}
+
+			rb_metadata_save (db->priv->metadata, &error);
+			if (error != NULL) {
+				g_warning ("error saving metadata to %s: %s", location,
+					   error->message);
+				/* FIXME */
+				g_free (location);
+				break;
+			}
+				
+			g_free (location);
 			break;
 		}
-		g_value_unset (&action->value);
+		}
 		g_free (action);
 	}
 
@@ -976,6 +1188,7 @@ rhythmdb_entry_set (RhythmDB *db, RhythmDBEntry *entry,
 		    guint propid, GValue *value)
 {
 	RhythmDBClass *klass = RHYTHMDB_GET_CLASS (db);
+	struct RhythmDBEntryChangeData *changedata;
 
 #ifndef G_DISABLE_ASSERT	
 	switch (G_VALUE_TYPE (value))
@@ -986,6 +1199,7 @@ rhythmdb_entry_set (RhythmDB *db, RhythmDBEntry *entry,
 	case G_TYPE_BOOLEAN:
 	case G_TYPE_INT:
 	case G_TYPE_LONG:
+	case G_TYPE_UINT64:
 	case G_TYPE_FLOAT:
 	case G_TYPE_DOUBLE:
 		break;
@@ -997,10 +1211,23 @@ rhythmdb_entry_set (RhythmDB *db, RhythmDBEntry *entry,
 
 	db_enter (db, TRUE);
 
-	if (!db->priv->changed_entries)
-		db->priv->changed_entries = g_hash_table_new (NULL, NULL);
+	changedata = g_new0 (struct RhythmDBEntryChangeData, 1);
+	changedata->entry = entry;
+	changedata->prop = propid;
 
-	g_hash_table_insert (db->priv->changed_entries, entry, NULL);
+	/* Copy a temporary gvalue, since _entry_get uses
+	 * _set_static_string to avoid memory allocations. */
+	{
+		GValue tem = {0,};
+		g_value_init (&tem, G_VALUE_TYPE (value));
+		klass->impl_entry_get (db, entry, propid, &tem);
+		g_value_init (&changedata->old, G_VALUE_TYPE (value));
+		g_value_copy (&tem, &changedata->old);
+		g_value_unset (&tem);
+	}
+	g_value_init (&changedata->new, G_VALUE_TYPE (value));
+	g_value_copy (value, &changedata->new);
+	db->priv->changed_entries = g_list_append (db->priv->changed_entries, changedata);
 
 	klass->impl_entry_set (db, entry, propid, value);
 
@@ -1127,7 +1354,7 @@ rhythmdb_query_copy (GPtrArray *array)
 	return ret;
 }
 
-GPtrArray *
+static GPtrArray *
 rhythmdb_query_parse_valist (RhythmDB *db, va_list args)
 {
 	RhythmDBQueryType query;
@@ -1280,8 +1507,18 @@ rhythmdb_nice_elt_name_from_propid (RhythmDB *db, gint propid)
 		return "last-played";
 	case RHYTHMDB_PROP_LAST_PLAYED_STR:
 		return "last-played-str";
-	case RHYTHMDB_PROP_QUALITY:
-		return "quality";
+	case RHYTHMDB_PROP_BITRATE:
+		return "bitrate";
+	case RHYTHMDB_PROP_TRACK_GAIN:
+		return "replaygain-track-gain";
+	case RHYTHMDB_PROP_TRACK_PEAK:
+		return "replaygain-track-peak";
+	case RHYTHMDB_PROP_ALBUM_GAIN:
+		return "replaygain-album-gain";
+	case RHYTHMDB_PROP_ALBUM_PEAK:
+		return "replaygain-album-peak";
+	case RHYTHMDB_PROP_MIMETYPE:
+		return "mimetype";
 	default:
 		g_assert_not_reached ();
 	}
@@ -1320,6 +1557,9 @@ write_encoded_gvalue (xmlNodePtr node,
 	case G_TYPE_LONG:
 		strval = g_strdup_printf ("%ld", g_value_get_long (val));
 		break;
+	case G_TYPE_UINT64:
+		strval = g_strdup_printf ("%llu", g_value_get_uint64 (val));
+		break;
 	case G_TYPE_FLOAT:
 		strval = g_strdup_printf ("%f", g_value_get_float (val));
 		break;
@@ -1364,11 +1604,14 @@ read_encoded_property (RhythmDB *db,
 	case G_TYPE_LONG:
 		g_value_set_long (val, g_ascii_strtoull (content, NULL, 10));
 		break;
+	case G_TYPE_UINT64:
+		g_value_set_uint64 (val, g_ascii_strtoull (content, NULL, 10));
+		break;
 	case G_TYPE_FLOAT:
 		g_value_set_float (val, g_ascii_strtod (content, NULL));
 		break;
 	case G_TYPE_DOUBLE:
-		g_value_set_float (val, g_ascii_strtod (content, NULL));
+		g_value_set_double (val, g_ascii_strtod (content, NULL));
 		break;
 	default:
 		g_assert_not_reached ();
@@ -1584,7 +1827,7 @@ rhythmdb_do_full_query_internal (RhythmDB *db, GtkTreeModel *main_model,
 	g_object_ref (G_OBJECT (main_model));
 	db->priv->outstanding_threads++;
 	query_thread_main (data, db);
-	while (rhythmdb_model_poll (RHYTHMDB_MODEL (main_model), NULL))
+	while (rhythmdb_query_model_poll (RHYTHMDB_QUERY_MODEL (main_model), NULL))
 		;
 }
 
@@ -1665,19 +1908,24 @@ rhythmdb_prop_get_type (void)
 		{
 
 			ENUM_ENTRY (RHYTHMDB_PROP_TYPE, "Type of entry (gint)"),
-			ENUM_ENTRY (RHYTHMDB_PROP_TITLE, "Title (gchararray)"),
-			ENUM_ENTRY (RHYTHMDB_PROP_GENRE, "Genre (gchararray)"),
-			ENUM_ENTRY (RHYTHMDB_PROP_ARTIST, "Artist (gchararray)"),
-			ENUM_ENTRY (RHYTHMDB_PROP_ALBUM, "Album (gchararray)"),
-			ENUM_ENTRY (RHYTHMDB_PROP_TRACK_NUMBER, "Track Number (gint)"),
-			ENUM_ENTRY (RHYTHMDB_PROP_DURATION, "Duration (glong)"),
-			ENUM_ENTRY (RHYTHMDB_PROP_FILE_SIZE, "File Size (glong)"),
+			ENUM_ENTRY (RHYTHMDB_PROP_TITLE, "Title"),
+			ENUM_ENTRY (RHYTHMDB_PROP_GENRE, "Genre"),
+			ENUM_ENTRY (RHYTHMDB_PROP_ARTIST, "Artist"),
+			ENUM_ENTRY (RHYTHMDB_PROP_ALBUM, "Album"),
+			ENUM_ENTRY (RHYTHMDB_PROP_TRACK_NUMBER, "Track Number"),
+			ENUM_ENTRY (RHYTHMDB_PROP_DURATION, "Duration"),
+			ENUM_ENTRY (RHYTHMDB_PROP_FILE_SIZE, "File Size (guint64)"),
 			ENUM_ENTRY (RHYTHMDB_PROP_LOCATION, "Location (gchararray)"),
 			ENUM_ENTRY (RHYTHMDB_PROP_MTIME, "Modification time (glong)"),
-			ENUM_ENTRY (RHYTHMDB_PROP_RATING, "Rating (gint)"),
+			ENUM_ENTRY (RHYTHMDB_PROP_RATING, "Rating (gdouble)"),
 			ENUM_ENTRY (RHYTHMDB_PROP_PLAY_COUNT, "Play Count (gint)"),
 			ENUM_ENTRY (RHYTHMDB_PROP_LAST_PLAYED, "Last Played (glong)"),
-			ENUM_ENTRY (RHYTHMDB_PROP_QUALITY, "Quality (gint)"),
+			ENUM_ENTRY (RHYTHMDB_PROP_BITRATE, "Bitrate"),
+			ENUM_ENTRY (RHYTHMDB_PROP_TRACK_GAIN, "Replaygain track gain"),
+			ENUM_ENTRY (RHYTHMDB_PROP_TRACK_PEAK, "Replaygain track peak"),
+			ENUM_ENTRY (RHYTHMDB_PROP_ALBUM_GAIN, "Replaygain album pain"),
+			ENUM_ENTRY (RHYTHMDB_PROP_ALBUM_PEAK, "Replaygain album peak"),
+			ENUM_ENTRY (RHYTHMDB_PROP_MIMETYPE, "Mime Type (gchararray)"),
 			{ 0, 0, 0 }
 		};
 
@@ -1747,6 +1995,7 @@ rhythmdb_entry_get_ ## NAME (RhythmDB *db, RhythmDBEntry *entry, guint propid) \
 DEFINE_GETTER(boolean, gboolean, BOOLEAN, FALSE)
 DEFINE_GETTER(pointer, gpointer, POINTER, NULL)
 DEFINE_GETTER(long, long, LONG, 0)
+DEFINE_GETTER(uint64, guint64, UINT64, 0)
 DEFINE_GETTER(int, int, INT, 0)
 DEFINE_GETTER(double, double, DOUBLE, 0)
 DEFINE_GETTER(float, float, FLOAT, 0)
@@ -1774,115 +2023,6 @@ rhythmdb_emit_entry_deleted (RhythmDB *db, RhythmDBEntry *entry)
 	g_signal_emit (G_OBJECT (db), rhythmdb_signals[ENTRY_DELETED], 0, entry);
 }
 
-typedef struct
-{
-	RhythmDB *db;
-	char *libname;
-} RhythmDBLegacyLoadData;
-
-RhythmDBEntry *
-rhythmdb_legacy_id_to_entry (RhythmDB *db, guint id)
-{
-	return g_hash_table_lookup (db->priv->legacy_id_map, GINT_TO_POINTER (id));
-}
-
-static gboolean
-emit_legacy_load_complete (RhythmDB *db)
-{
-	GDK_THREADS_ENTER ();
-
-	g_signal_emit (G_OBJECT (db), rhythmdb_signals[LEGACY_LOAD_COMPLETE], 0);
-	g_hash_table_destroy (db->priv->legacy_id_map);
-	g_object_unref (G_OBJECT (db));
-
-	GDK_THREADS_LEAVE ();
-
-	return FALSE;
-}
-
-gpointer
-legacy_load_thread_main (RhythmDBLegacyLoadData *data)
-{
-	xmlDocPtr doc;
-	xmlNodePtr root, child;
-	guint id;
-
-	doc = xmlParseFile (data->libname);
-
-	if (doc == NULL) {
-		g_object_unref (G_OBJECT (data->db));
-		goto free_exit;
-	}
-
-	rb_debug ("parsing entries");
-	root = xmlDocGetRootElement (doc);
-	for (child = root->children; child != NULL; child = child->next) {
-		RhythmDBEntry *entry = 
-			rhythmdb_legacy_parse_rbnode (data->db, RHYTHMDB_ENTRY_TYPE_SONG, child,
-						      &id);
-
-		if (id > 0)
-			g_hash_table_insert (data->db->priv->legacy_id_map, GINT_TO_POINTER (id),
-					     entry);
-		if (entry) {
-			rhythmdb_entry_ref (data->db, entry);
-			g_async_queue_push (data->db->priv->update_queue, entry);
-		}
-	}
-	xmlFreeDoc (doc);
-
-	/* steals the library ref */
-	g_idle_add ((GSourceFunc) emit_legacy_load_complete, data->db);
-	rb_debug ("legacy load thread exiting");
-free_exit:
-	g_free (data->libname);
-	g_free (data);
-	g_thread_exit (NULL);
-	return NULL;
-}
-
-void
-rhythmdb_load_legacy (RhythmDB *db)
-{
-	RhythmDBLegacyLoadData *data;
-	char *libname = g_build_filename (rb_dot_dir (), "library-2.1.xml", NULL);
-	xmlDocPtr doc;
-	xmlNodePtr root, child;
-
-	if (!g_file_test (libname, G_FILE_TEST_EXISTS)) {
-		g_free (libname);
-		goto load_iradio;
-	}
-
-	data = g_new0 (RhythmDBLegacyLoadData, 1);
-	data->db = db;
-	g_object_ref (G_OBJECT (data->db));
-	data->libname = libname;
-	
-	rb_debug ("kicking off library legacy loading thread");
-	g_thread_create ((GThreadFunc) legacy_load_thread_main, data, FALSE, NULL);
-
-load_iradio:
-	libname = g_build_filename (rb_dot_dir (), "iradio-2.2.xml", NULL);
-	if (!g_file_test (libname, G_FILE_TEST_EXISTS)) {
-		g_free (libname);
-		return;
-	}
-
-	doc = xmlParseFile (libname);
-	g_free (libname);
-
-	if (doc == NULL)
-		return;
-
-	root = xmlDocGetRootElement (doc);
-	for (child = root->children; child != NULL; child = child->next) {
-		rhythmdb_legacy_parse_rbnode (db, RHYTHMDB_ENTRY_TYPE_IRADIO_STATION,
-					      child, NULL);
-	}
-	xmlFreeDoc (doc);
-}
-
 static gboolean
 queue_is_empty (GAsyncQueue *queue)
 {
@@ -1905,40 +2045,44 @@ rhythmdb_get_status (RhythmDB *db)
 char *
 rhythmdb_compute_status_normal (gint n_songs, glong duration, GnomeVFSFileSize size)
 {
-	float days;
-	long hours, minutes, seconds;
+	long days, hours, minutes, seconds;
 	char *songcount;
 	char *time;
 	char *size_str;
 	char *ret;
+	const char *minutefmt;
+	const char *hourfmt;	
+	const char *dayfmt;
 
 	songcount = g_strdup_printf (ngettext ("%d song", "%d songs", n_songs), n_songs);
 
-	days    = (float) duration / (float) (60 * 60 * 24); 
-	hours   = duration / (60 * 60);
-	minutes = duration / 60 - hours * 60;
+	days    = duration / (60 * 60 * 24); 
+	hours   = (duration / (60 * 60)) - (days * 24);
+	minutes = (duration / 60) - ((days * 24 * 60) + (hours * 60));
 	seconds = duration % 60;
 
-	if (days >= 1.0) {
-		time = ngettext ("%.1f day", "%.1f days", days);
-		time = g_strdup_printf (time, days);			
+	minutefmt = ngettext ("%ld minute", "%ld minutes", minutes);
+	hourfmt = ngettext ("%ld hour", "%ld hours", hours);
+	dayfmt = ngettext ("%ld day", "%ld days", days);
+	if (days >= 1) {
+		char *fmt;
+		/* Translators: the format is "X days, X hours and X minutes" */
+		fmt = g_strdup_printf (_("%s, %s and %s"), dayfmt, hourfmt, minutefmt);
+		time = g_strdup_printf (fmt, days, hours, minutes);
+		g_free (fmt);
 	} else {
-		const char *minutefmt = ngettext ("%ld minute", "%ld minutes", minutes);
 		if (hours >= 1) {		
 			const char *hourfmt = ngettext ("%ld hour", "%ld hours", hours);
 			char *fmt;
-			if (minutes > 0) {
-				/* Translators: the format is "X hours and X minutes" */
-				fmt = g_strdup_printf (_("%s and %s"), hourfmt, minutefmt);
-			} else {
-				/* Translators: the format is "X hours" */
-				fmt = g_strdup_printf ("%s", hourfmt);
-			}
+			/* Translators: the format is "X hours and X minutes" */
+			fmt = g_strdup_printf (_("%s and %s"), hourfmt, minutefmt);
 			time = g_strdup_printf (fmt, hours, minutes);
 			g_free (fmt);
-		} else 
+		} else {
 			time = g_strdup_printf (minutefmt, minutes);
+		}
 	}
+
 	size_str = gnome_vfs_format_file_size_for_display (size);
 	ret = g_strdup_printf ("%s, %s, %s", songcount, time, size_str);
 	g_free (songcount);
