@@ -68,6 +68,9 @@ static void rb_playlist_manager_cmd_new_automatic_playlist (BonoboUIComponent *c
 static void rb_playlist_manager_cmd_delete_playlist (BonoboUIComponent *component,
 						     RBPlaylistManager *mgr,
 						     const char *verbname);
+static void rb_playlist_manager_cmd_edit_automatic_playlist (BonoboUIComponent *component,
+							     RBPlaylistManager *mgr,
+							     const char *verbname);
 static void handle_playlist_entry_into_playlist_cb (RBPlaylist *playlist, const char *uri, const char *title,
 						    const char *genre, RBPlaylistManager *mgr);
 
@@ -123,6 +126,7 @@ static BonoboUIVerb rb_playlist_manager_verbs[] =
 	BONOBO_UI_VERB ("SavePlaylist", (BonoboUIVerbFn) rb_playlist_manager_cmd_save_playlist),
 	BONOBO_UI_VERB ("FileDeletePlaylist",(BonoboUIVerbFn) rb_playlist_manager_cmd_delete_playlist),
 	BONOBO_UI_VERB ("DeletePlaylist",  (BonoboUIVerbFn) rb_playlist_manager_cmd_delete_playlist),
+ 	BONOBO_UI_VERB ("EditAutomaticPlaylist",  (BonoboUIVerbFn) rb_playlist_manager_cmd_edit_automatic_playlist),
 	BONOBO_UI_VERB_END
 };
 
@@ -440,117 +444,6 @@ append_new_playlist_source (RBPlaylistManager *mgr, RBPlaylistSource *source)
 		       source);
 }
 
-static void
-rb_playlist_manager_load_legacy_playlist (RBPlaylistManager *mgr,
-					  xmlNodePtr root,
-					  RBPlaylistSource *source)
-{
-	xmlNodePtr child;
-	for (child = root->children; child != NULL; child = child->next) {
-		long id;
-		char *tmp;
-		RhythmDBEntry *entry;
-		const char *location;
-
-		tmp = xmlGetProp (child, "id");
-		if (tmp == NULL)
-			continue;
-		id = atol (tmp);
-		g_free (tmp);
-
-		entry = rhythmdb_legacy_id_to_entry (mgr->priv->db, id);
-		if (!entry) {
-			rb_debug ("invalid legacy id %d", id);
-			continue;
-		}
-
-		location = rhythmdb_entry_get_string (mgr->priv->db, entry,
-						      RHYTHMDB_PROP_LOCATION);
-
-		rb_playlist_source_add_location (source, location);
-	}
-}
-
-void
-rb_playlist_manager_load_legacy_playlists (RBPlaylistManager *mgr)
-{
-	char *oldpath, *path;
-	GnomeVFSDirectoryHandle *handle;
-	GnomeVFSResult result;
-	GnomeVFSFileInfo *info;
-
-	/* Backwards compatibility with old "group" name */
-	oldpath = g_build_filename (rb_dot_dir (), "groups", NULL);
-	path = g_build_filename (rb_dot_dir (), "playlists", NULL);
-
-	if ((result = gnome_vfs_directory_open (&handle, oldpath, GNOME_VFS_FILE_INFO_FOLLOW_LINKS))
-	    == GNOME_VFS_OK) {
-		rb_debug ("Renaming legacy group dir");
-		gnome_vfs_move (oldpath, path, TRUE);
-	}
-
-	g_free (oldpath);
-
-	if ((result = gnome_vfs_directory_open (&handle, path, GNOME_VFS_FILE_INFO_FOLLOW_LINKS))
-	    != GNOME_VFS_OK)
-		goto out;
-
-	info = gnome_vfs_file_info_new ();
-	while ((result = gnome_vfs_directory_read_next (handle, info)) == GNOME_VFS_OK) {
-		RBPlaylistSource *playlist;
-		RhythmDBQueryModel *model;
-		char *filepath, *name, *xml;
-		xmlDocPtr doc;
-		xmlNodePtr root;
-
-		if (info->name[0] == '.')
-			continue;
-
-		filepath = g_build_filename (path, info->name, NULL);
-
-		doc = xmlParseFile (filepath);
-		
-		if (doc == NULL)
-			continue;
-
-		root = xmlDocGetRootElement (doc);
-		xml = xmlGetProp (root, "version");
-		if (xml == NULL || strcmp (xml, "1.0") != 0) {
-			g_free (xml);
-			xmlFreeDoc (doc);
-			continue;
-		}
-		g_free (xml);
-
-		name = xmlGetProp (root, "name");
-		if (name == NULL) {
-			xmlFreeDoc (doc);
-			continue;
-		}
-
-		rhythmdb_read_lock (mgr->priv->db);
-
-		playlist = RB_PLAYLIST_SOURCE (rb_playlist_source_new (mgr->priv->db, FALSE));
-		g_object_set (G_OBJECT (playlist), "name", name, NULL);
-		g_free (name);
-
-		model = rb_playlist_source_get_model (playlist);
-		
-		rb_playlist_manager_load_legacy_playlist (mgr, root,
-							  playlist);
-
-		append_new_playlist_source (mgr, playlist);
-
-		rhythmdb_read_unlock (mgr->priv->db);
-		xmlFreeDoc (doc);
-		g_free (filepath);
-	}
-
-	gnome_vfs_file_info_unref (info);
-out:
-	g_free (path);
-}
-
 void
 rb_playlist_manager_load_playlists (RBPlaylistManager *mgr)
 {
@@ -612,10 +505,11 @@ rb_playlist_manager_save_playlists (RBPlaylistManager *mgr)
 }
 
 RBSource *
-rb_playlist_manager_new_playlist (RBPlaylistManager *mgr, gboolean automatic)
+rb_playlist_manager_new_playlist (RBPlaylistManager *mgr,
+				  const char *suggested_name, gboolean automatic)
 {
 	RBSource *playlist = RB_SOURCE (rb_playlist_source_new (mgr->priv->db, automatic));
-	g_object_set (G_OBJECT (playlist), "name", "", NULL);
+	g_object_set (G_OBJECT (playlist), "name", suggested_name ? suggested_name : "", NULL);
 	append_new_playlist_source (mgr, RB_PLAYLIST_SOURCE (playlist));
 	rb_sourcelist_edit_source_name (mgr->priv->sourcelist, playlist);
 	return playlist;
@@ -626,7 +520,25 @@ rb_playlist_manager_cmd_new_playlist (BonoboUIComponent *component,
 				      RBPlaylistManager *mgr,
 				      const char *verbname)
 {
-	rb_playlist_manager_new_playlist (mgr, FALSE);
+	rb_playlist_manager_new_playlist (mgr, NULL, FALSE);
+}
+
+static void
+rb_playlist_manager_set_automatic_playlist (RBPlaylistManager *mgr,
+					    RBPlaylistSource *playlist,
+					    RBQueryCreator *creator)
+{
+	RBQueryCreatorLimitType type;
+	guint limit, limit_count = 0, limit_size = 0;
+
+	rb_query_creator_get_limit (creator, &type, &limit);
+	if (type == RB_QUERY_CREATOR_LIMIT_COUNT)
+		limit_count = limit;
+	if (type == RB_QUERY_CREATOR_LIMIT_MB)
+		limit_size = limit;
+	rb_playlist_source_set_query (playlist,
+				      rb_query_creator_get_query (creator),
+				      limit_count, limit_size);
 }
 
 static void
@@ -636,8 +548,6 @@ rb_playlist_manager_cmd_new_automatic_playlist (BonoboUIComponent *component,
 {
 	RBQueryCreator *creator = RB_QUERY_CREATOR (rb_query_creator_new (mgr->priv->db));
 	RBSource *playlist;
-	RBQueryCreatorLimitType type;
-	guint limit, limit_count = 0, limit_size = 0;
 	
 	switch (gtk_dialog_run (GTK_DIALOG (creator)))
 	{
@@ -646,16 +556,35 @@ rb_playlist_manager_cmd_new_automatic_playlist (BonoboUIComponent *component,
 		gtk_widget_destroy (GTK_WIDGET (creator));	
 		return;
 	}
+	playlist = rb_playlist_manager_new_playlist (mgr, NULL, TRUE);
 
-	rb_query_creator_get_limit (creator, &type, &limit);
-	if (type == RB_QUERY_CREATOR_LIMIT_COUNT)
-		limit_count = limit;
-	if (type == RB_QUERY_CREATOR_LIMIT_MB)
-		limit_size = limit;
-	playlist = rb_playlist_manager_new_playlist (mgr, TRUE);
-	rb_playlist_source_set_query (RB_PLAYLIST_SOURCE (playlist),
-				      rb_query_creator_get_query (creator),
-				      limit_count, limit_size);
+	rb_playlist_manager_set_automatic_playlist (mgr, RB_PLAYLIST_SOURCE (playlist), creator); 
+
+	gtk_widget_destroy (GTK_WIDGET (creator));	
+}
+
+static void
+rb_playlist_manager_cmd_edit_automatic_playlist (BonoboUIComponent *component,
+						 RBPlaylistManager *mgr,
+						 const char *verbname)
+{
+	RBQueryCreator *creator;
+	RBPlaylistSource *playlist;
+	GPtrArray *query;
+	guint limit_count = 0, limit_size = 0;
+	
+
+	playlist = RB_PLAYLIST_SOURCE (mgr->priv->selected_source);
+	rb_playlist_source_get_query (playlist, &query, &limit_count, &limit_size);
+
+	creator = RB_QUERY_CREATOR (rb_query_creator_new_from_query (mgr->priv->db,
+								     query,
+								     limit_count,
+								     limit_size));	
+
+	gtk_dialog_run (GTK_DIALOG (creator));
+
+	rb_playlist_manager_set_automatic_playlist (mgr, playlist, creator);
 	gtk_widget_destroy (GTK_WIDGET (creator));	
 }
 
@@ -732,23 +661,6 @@ save_playlist_response_cb (GtkDialog *dialog,
 	g_free (file);
 }
 
-/* static gboolean */
-/* rb_playlist_manager_validate_name (RBPlaylistManager *mgr, const char *name, gboolean error_dialog) */
-/* { */
-/* 	GList *tem; */
-/* 	char *temname; */
-
-/* 	for (tem = mgr->priv->playlists; tem; tem = g_list_next (tem)) { */
-/* 		g_object_get (G_OBJECT (tem->data), "name", &temname, NULL); */
-/* 		if (!strcmp (temname, name)) { */
-/* 			if (error_dialog) */
-/* 				rb_error_dialog (_("There is already a playlist with that name.")); */
-/* 			return FALSE; */
-/* 		} */
-/* 	} */
-/* 	return TRUE; */
-/* } */
-
 static void
 rb_playlist_manager_cmd_save_playlist (BonoboUIComponent *component,
 				       RBPlaylistManager *mgr,
@@ -801,7 +713,7 @@ handle_playlist_entry_into_playlist_cb (RBPlaylist *playlist, const char *uri, c
 
 	if (!mgr->priv->loading_playlist) {
 		mgr->priv->loading_playlist =
-			RB_PLAYLIST_SOURCE (rb_playlist_manager_new_playlist (mgr, FALSE));
+			RB_PLAYLIST_SOURCE (rb_playlist_manager_new_playlist (mgr, NULL, FALSE));
 	}
 	add_uri_to_playlist (mgr, mgr->priv->loading_playlist, uri, title);
 }
