@@ -1,4 +1,6 @@
 /*
+ *  arch-tag: Header for main Rhythmbox shell
+ *
  *  Copyright (C) 2002 Jorn Baayen
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -15,7 +17,6 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *  
- *  $Id: rb-shell.h,v 1.8 2003/08/01 04:48:54 cwalters Exp $
  */
 
 #ifndef __RB_SHELL_H
@@ -54,11 +55,13 @@ typedef struct
         POA_GNOME_Rhythmbox__epv epv;
 } RBShellClass;
 
-GType		rb_shell_get_type  (void);
+GType		rb_shell_get_type	(void);
 
-RBShell *	rb_shell_new       (void);
+RBShell *	rb_shell_new		(int argc, char **argv,
+					 gboolean no_registration, gboolean dry_run,
+					 char *rhythmdb);
 
-void		rb_shell_construct (RBShell *shell);
+void		rb_shell_construct	(RBShell *shell);
 
 /* utilities */
 
