@@ -31,8 +31,6 @@
 
 #include "rb-iradio-source.h"
 
-#include "rhythmdb-legacy.h"
-#include "rhythmdb-model.h"
 #include "rhythmdb-query-model.h"
 #include "rb-bonobo-helpers.h"
 #include "rb-glade-helpers.h"
@@ -303,6 +301,7 @@ rb_iradio_source_constructor (GType type, guint n_construct_properties,
 						    FALSE, FALSE);
 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_TITLE);
 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_GENRE);
+	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_QUALITY);
 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_RATING);
 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_PLAY_COUNT);
 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_LAST_PLAYED);
@@ -414,15 +413,25 @@ rb_iradio_source_add_station (RBIRadioSource *source,
 	if (!entry)
 		entry = rhythmdb_entry_new (source->priv->db, RHYTHMDB_ENTRY_TYPE_IRADIO_STATION, uri);
 	g_value_init (&val, G_TYPE_STRING);
-	g_value_set_string (&val, title);
+	if (title)
+		g_value_set_string (&val, title);
+	else
+		g_value_set_string (&val, uri);
+
 	rhythmdb_entry_set (source->priv->db, entry, RHYTHMDB_PROP_TITLE, &val);
+	g_value_reset (&val);
+		
 	if (genre) {
-		g_value_reset (&val);
 		g_value_set_string (&val, genre);
 		rhythmdb_entry_set (source->priv->db, entry, RHYTHMDB_PROP_GENRE, &val);
 	}
 	g_value_unset (&val);
-		
+	
+	g_value_init (&val, G_TYPE_DOUBLE);
+	g_value_set_double (&val, 3.0);
+	rhythmdb_entry_set (source->priv->db, entry, RHYTHMDB_PROP_RATING, &val);
+	g_value_unset (&val);
+	
 	rhythmdb_write_unlock (source->priv->db);
 }
 
@@ -453,6 +462,8 @@ impl_search (RBSource *asource, const char *search_text)
 	else
 		source->priv->search_text = NULL;
 	rb_iradio_source_do_query (source, RB_IRADIO_QUERY_TYPE_SEARCH);
+
+	rb_source_notify_filter_changed (RB_SOURCE (source));
 }
 
 static RBEntryView *
@@ -601,6 +612,7 @@ rb_iradio_source_songs_view_sort_order_changed_cb (RBEntryView *view,
 						   RBIRadioSource *source)
 {
 	rb_debug ("sort order changed");
+	rb_entry_view_set_resorting (view);
 	rb_iradio_source_do_query (source, RB_IRADIO_QUERY_TYPE_SEARCH);
 }
 
@@ -633,6 +645,8 @@ genre_selected_cb (RBPropertyView *propview, const char *name,
 	g_free (iradio_source->priv->selected_genre);
 	iradio_source->priv->selected_genre = g_strdup (name);
 	rb_iradio_source_do_query (iradio_source, RB_IRADIO_QUERY_TYPE_GENRE);
+
+	rb_source_notify_filter_changed (RB_SOURCE (iradio_source));
 }
 
 static void
@@ -711,7 +725,7 @@ rb_iradio_source_do_query (RBIRadioSource *source, RBIRadioQueryType qtype)
 
 	model = GTK_TREE_MODEL (query_model);
 	
-	rb_entry_view_set_model (source->priv->stations, RHYTHMDB_MODEL (query_model));
+	rb_entry_view_set_model (source->priv->stations, RHYTHMDB_QUERY_MODEL (query_model));
 
 	rhythmdb_do_full_query_parsed (source->priv->db, model, query);
 
